/*================================================================== 
 * Licensed Materials - Property of IBM 
 * IBM SecureWay On Demand Client/Server 
 * (c) Copyright IBM Corp. 1999  All rights reserved. 
 * 
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with IBM 
 * Corp. 
 * 
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is 
 * sample code created by IBM Corporation.  This sample code is 
 * not part of any standard or IBM product and is provided to you 
 * solely for the purpose of assisting you in the development of 
 * your applications.  The code is provided "AS IS", without 
 * warranty of any kind.  IBM shall not be liable for any damages 
 * arising out of your use of the sample code, even if they have 
 * been advised of the possibility of such damages. 
 ******************************************************************** 
 * AdminServer.java
 *  
 * Demo: Pager Messaging 
 *
 * Purpose:
 * To give the MessageNow applet a listing of all of the users,
 * or user groups in the On-Demand Server.
 ******************************************************************** 
 * Change History: 
 ******************************************************************** 
 * 
 *  REASON   DATE     ORIGIN     DESCRIPTION 
 *  -------- ------   ------     ----------- 
 *  new      052599   Steve Ims  New code 
 *===================================================================*/ 

package Demos.MessageDemo.MessageNow;


import java.io.*;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

import com.ibm.managed.preference.*;
import com.ibm.managed.resource.*;
import com.ibm.managed.security.*;
import com.ibm.managed.security.client.*;


/**
 * <p>This Servlet class runs as an administrator in order to
 * give the MessageNow applet access to the list of users
 * and user groups.  This information is not directly available
 * to user classes.
 * 
 * <p>The aspect of this servlet unique to On Demand Server is 
 * the use of the <b>Domain</b> class' <em>list</em> method to
 * obtain enumerations of various resources (In this case, 
 * an enumeration of all of the users or all of the user groups.)
 * Criteria could also be used to obtain a more selective 
 * listing of resources.
 * 
 * <p>The user OAT associated with the user of the MessageNow
 * applet is also passed to this class.  It does not make sense
 * for this particular demo to actually make use of this OAT.  
 * However, in other instances it may be useful to have the 
 * servlet attempt to use the OAT passed to it to perform
 * the functions of the servlet.  This allows the servlet to pass 
 * the duties of access control onto On Demand Server, which 
 * can then determine if a certain user should be allowed to 
 * perform the functions carried out by the servlet.
 * 
 * @author Steve Ims
 */
public class AdminServer extends HttpServlet {
    protected LoginContext               _adminLC;
    protected Domain                 _adminDomain;

    // Commands
    public final static int ALL_USERS       = 0;
    public final static int ALL_USER_GROUPS = 1;


    public void init(ServletConfig config) 
        throws ServletException
    {
        super.init(config);

        // Default login context is "administrator" because ODS has
        // already setup default login.
        try {
            _adminLC = LoginContext.getDefaultLoginContext();
            _adminDomain = DomainFactory.getDomain(_adminLC);

        } catch(ResourceException x) {
            x.printStackTrace();
        }
    }


    public void service(HttpServletRequest req, HttpServletResponse res)
        throws ServletException, IOException
    {
        // Setup streams for handling input and output.
        ObjectInputStream ois = new ObjectInputStream(req.getInputStream());
        ObjectOutputStream oos = new ObjectOutputStream(res.getOutputStream());


        try {
            // Get input data from the client.
            OAT userOAT = (OAT) ois.readObject();
            int type = ois.readInt();


            Enumeration e;

            // Here you could try to use the user OAT to attempt to obtain
            //  the list of users or groups.  However, if the user is not
            //  an administrator, this demo will not work.
            // Therefore, we use the servlets Administrative OAT in order
            //  to obtain the list of users.
            
            switch(type) {
                                
            case ALL_USERS:
                // use the administrative context to obtain an enumeration
                //  of all of the users on the Server            
                e = _adminDomain.list(ResourceTypes.USER, null, null, null);
                oos.writeObject(convertEnumerationToVector(e));
                break;

            case ALL_USER_GROUPS:
                // use the administrative context to obtain an enumeration
                //  of all of the user groups on the Server                
                
                e = _adminDomain.list(ResourceTypes.USER_GROUP, 
                                      null, null, null);
                oos.writeObject(convertEnumerationToVector(e));
                break;
            }

        } catch(Exception x) {
            x.printStackTrace();
            oos.writeObject(null);

        } finally {
            oos.close();
        }
    }



    /**
     * Enumeration objects are not Serializable, so pass the information
     * as a Vector instead.
     * 
     * @param e
     */
    static Vector convertEnumerationToVector(Enumeration e) {
        Vector v = new Vector();

        while(e.hasMoreElements()) {
            v.addElement(e.nextElement());
        }

        return(v);
    }
}
