/*================================================================== 
 * Licensed Materials - Property of IBM 
 * IBM SecureWay On Demand Client/Server 
 * (c) Copyright IBM Corp. 1997  All rights reserved. 
 * 
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with IBM 
 * Corp. 
 * 
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is 
 * sample code created by IBM Corporation.  This sample code is 
 * not part of any standard or IBM product and is provided to you 
 * solely for the purpose of assisting you in the development of 
 * your applications.  The code is provided "AS IS", without 
 * warranty of any kind.  IBM shall not be liable for any damages 
 * arising out of your use of the sample code, even if they have 
 * been advised of the possibility of such damages. 
 ******************************************************************** 
 * MessageClient.java
 *  
 * Demo: Pager Messaging 
 *
 * Purpose:
 * Encapsulate network communication with MessageServer Servlet
 *
 ******************************************************************** 
 * Change History: 
 ******************************************************************** 
 * 
 *  REASON   DATE     ORIGIN     DESCRIPTION 
 *  -------- ------   ------     ----------- 
 *  new      052599   Steve Ims  New code 
 *===================================================================*/ 

package Demos.MessageDemo.MessageServer.client;

import java.io.*;
import java.net.*;

import Demos.MessageDemo.MessageServer.*;


/**
 * <p>This class encapsulates the network communication between the MessageNow
 * applet, and the MessageServer Servlet.  One of two methods, <em> sendMessageToUser </em>
 * or <em> sendMessageToGroup </em>, is used depending on whether the recipient is a 
 * group or an individual user.
 * 
 * @author Steve Ims
 * @see MessageNow
 * @see MessageServer
 * @see MessageConstants
 */
public class MessageClient {
    String  _serverHostName;
    int     _serverPort;


    /**
     * Construct an object to connect to messageServerHostName on 
     *  port messageServerPort.
     * 
     * @param messageServerHostName The name of the server on which the 
     *      MessageServer Servlet is running.
     * @param messageServerPort The port number on which the MessageServer
     *      Servlet is listening.
     */
    public MessageClient(String messageServerHostName, int messageServerPort) {
        _serverHostName = new String(messageServerHostName);
        _serverPort     = messageServerPort;
    }


    /**
     * Sends a message to an individual user.
     * 
     * @param userId The name of the individual recipient
     * @param message The actual text of the message
     *
     * @return a MessageResponse object which contains the result
     * of the attempted delivery of the message to
     *
     * @see sendMessageToServer
     */
    public MessageResponse sendMessageToUser(String userId, String message)
    {
        return(sendMessageToServer(MessageConstants.RECIPIENT_USER, 
                                   userId, message));
    }
    
    /**
     * Sends a message to all of the users within the group
     * specified.
     * 
     * @param groupName The name of the group that the message is intended for
     * @param message The actual text of the message
     *
     * @return a MessageResponse object which contains the result
     * of the attempted delivery of the message to
     *
     * @see sendMessageToServer
     */
    public MessageResponse sendMessageToGroup(String groupName, 
                                              String message) 
    {
        return(sendMessageToServer(MessageConstants.RECIPIENT_USER_GROUP, 
                                   groupName, message));
    }


    /**
     * This function does all the real network communication.
     * 
     * Makes three attempts to connect to the servlet 
     * before it issues an error message.
     * 
     * @param type Flag that specifies whether the recipient is an 
     *      individual user or a group.  It can be either:
     *      MessageConstants.RECIPIENT_USER or
     *      MessageConstants.RECIPIENT_USER_GROUP
     * @param recipient The name of the recipient, can either be the 
     *      name of an individual, or a group.
     * @param message The actual text of the message.
     *
     * @return a MessageResponse object which contains the result
     *      of the attempted delivery of the message to
     */
    MessageResponse sendMessageToServer(int     type, 
                                        String  recipient, 
                                        String  message) 
    {
        try {
            // construct URL to connect to MessageServer servlet
            URL url = new URL("http://" + _serverHostName 
                              + ":" + _serverPort 
                              + MessageConstants.MESSAGE_SERVER_SERVLET_URL);

            // Make no more than three attempts to connect to servlet
            int remainingRetries = 3;
            URLConnection conn = null;

            do {
                try {
                    conn = url.openConnection();
                    conn.setUseCaches(false);
                    conn.setDoOutput(true);
                    conn.setDoInput(true);

                    ObjectOutputStream oos 
                        = new ObjectOutputStream(conn.getOutputStream());
                    oos.writeInt(type);
                    oos.writeObject(recipient);
                    oos.writeObject(message);

                    oos.close();
                    remainingRetries = 0;

                } catch(IOException x) {
                    remainingRetries--;
                    if(remainingRetries <= 0) {
                        return(new MessageResponse(false, "Failed to connect!!!"));
                    }
                }

            } while(remainingRetries > 0);

            ObjectInputStream ois = new ObjectInputStream(conn.getInputStream());

            // read the response code from the serverlet
            int responseType = ois.readInt();

            // take appropriate action based on response code
            switch(responseType) {
            case MessageConstants.SERVER_RESPONSE_SUCCESS:
                return(new MessageResponse(true, 
                                           (String []) ois.readObject()));

            case MessageConstants.SERVER_RESPONSE_EXCEPTION:
                return(new MessageResponse(false, (String) ois.readObject()));

            default:
                return(new MessageResponse(false, 
                                           "Unknown response from servlet = " 
                                           + responseType));
            }

        } catch(Exception x) {
            return(new MessageResponse(false, "Exception: " + x.toString()));
        }
    }
}
