/*================================================================== 
 * Licensed Materials - Property of IBM 
 * IBM SecureWay On Demand Client/Server 
 * (c) Copyright IBM Corp. 1999  All rights reserved. 
 * 
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with IBM 
 * Corp. 
 * 
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is 
 * sample code created by IBM Corporation.  This sample code is 
 * not part of any standard or IBM product and is provided to you 
 * solely for the purpose of assisting you in the development of 
 * your applications.  The code is provided "AS IS", without 
 * warranty of any kind.  IBM shall not be liable for any damages 
 * arising out of your use of the sample code, even if they have 
 * been advised of the possibility of such damages. 
 ******************************************************************** 
 * MessageServer.java
 *
 * Demo: Pager Messaging 
 *
 * Purpose:
 * This is the Administrative Servlet in charge of determining
 * the preferred method of delivery for each recipient, and 
 * delivering the actual message.  This demo actually only 
 * implements paging.
 *
 ******************************************************************** 
 * Change History: 
 ******************************************************************** 
 * 
 *  REASON   DATE     ORIGIN     DESCRIPTION 
 *  -------- ------   ------     ----------- 
 *  new      052599   Steve Ims  New code 
 *===================================================================*/ 

package Demos.MessageDemo.MessageServer.server;

import java.io.*;
import java.net.*;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

import com.ibm.managed.preference.*;
import com.ibm.managed.resource.*;
import com.ibm.managed.security.*;
import com.ibm.managed.security.client.*;

import Demos.MessageDemo.MessageServer.*;


/**
 * <p>This servlet class performs most of the key function
 * involved in the messaging demo.  The servlet obtains the 
 * recipient list from the message, determines the preferred 
 * method of delivery for each user, and then delivers the
 * message to each user.
 * 
 * <p> The aspects of this class unique to On-Demand Server 
 * include the following:
 * <ul>
 * 	<li>This use of the Administrative login context in order
 * 		to access information not available to the user applet.
 * 
 * 	<li>The use of the ODS <b>Domain</b> and <b>Resource</b> 
 * 		interfaces, along with the <b>PreferenceSet</b> class
 * 		in order to obtain information about individual user preferences.
 * 
 * </ul>
 * 
 * @author Steve Ims
 * @see MessageNow
 * @see MessageResponse
 * @see PagerMessenger
 * @see SocksConfig
 */
public class MessageServer extends HttpServlet {
    protected LoginContext               _adminLC;
    protected Domain                 _adminDomain;


    /**
     * Method called when servlet is loaded.
     * 
     * @param config Not sure what the config parameter does???
     * @exception javax.servlet.ServletException
     */
    public void init(ServletConfig config) 
        throws ServletException
    {
        super.init(config);

        // Default login context is "administrator" because ODS has
        // already setup default login.
        try {
            _adminLC = LoginContext.getDefaultLoginContext();
            _adminDomain = DomainFactory.getDomain(_adminLC);

        } catch(ResourceException x) {
            x.printStackTrace();
        }
    }


    /**
     * Method called when servlet is contacted.
     * 
     * @param req
     * @param res
     * @exception javax.servlet.ServletException
     * @exception java.io.IOException
     */
    public void service(HttpServletRequest req, HttpServletResponse res)
        throws ServletException, IOException
    {
        // Setup streams for handling input and output.
        ObjectInputStream ois = new ObjectInputStream(req.getInputStream());
        ObjectOutputStream oos = new ObjectOutputStream(res.getOutputStream());


        try {

            // Get input data from the client.
            int type = ois.readInt();
            String recipientName = (String) ois.readObject();
            String message = (String) ois.readObject();


            System.out.println("type = " + type
                               + "; recipientName = " + recipientName
                               + "; message = " + message);


            // Build a vector of user names for recipient list.  Size
            // is unity for type == USER, but can be larger for
            // type == USER_GROUP.
            Vector recipientList = getRecipientList(type, recipientName);

        
            System.out.println("got recipientList = " 
                               + recipientList.toString());


            // Send the message to all recipients.  Requires that we lookup
            // each user's notification method (pager, email, etc).  Queue
            // response messages for return when complete.
            Vector responses = new Vector();
            Enumeration recipients = recipientList.elements();
            
            while(recipients.hasMoreElements()) {
                System.out.println("getting next object...");
                Object o = recipients.nextElement();
                System.out.println("class type = " + o.getClass().getName());
                Resource user = (Resource) o;
                
                // Send message to user via their preferred delivery method
                String response = sendMessageToUser(user.getName(), message);
                
                // Queue response
                responses.addElement(response);
            }


            System.out.println("done sending messages");
            System.out.println("responses = " + responses.toString());


            // Return the response messages as a String [].
            String [] responseMessages = new String[responses.size()];
            for(int i=0; i<responseMessages.length; i++) {
                responseMessages[i] = (String) responses.elementAt(i);
            }

            // Write back successful return code to client applet
            oos.writeInt(MessageConstants.SERVER_RESPONSE_SUCCESS);
            
            oos.writeObject(responseMessages);
            oos.close();

        } catch(Exception x) {
            oos.writeInt(MessageConstants.SERVER_RESPONSE_EXCEPTION);
            oos.writeObject(x.toString());

            x.printStackTrace();

        } finally {
            oos.close();
        }
    }

    /**
     * Obtains a list of the individual recipients to receive the 
     * message.
     * 
     * @param type The type of recipient list: either 
     *      MessageConstants.RECIPIENT_USER
     *      or MessageConstants.RECIPIENT_USER_GROUP
     * @param recipientName The name of the user or group that will 
     *      receive the message.
     *       
     * @return Returns a vector of User objects.  One element if
     *      type == RECIPIENT_USER, varied if type == RECIPIENT_USER_GROUP.
     *
     * @exception ResourceException
     */
    Vector getRecipientList(int type, String recipientName) 
        throws ResourceException
    {
        Vector list = new Vector();

        switch(type) {
            
            // recipient is one user, so return list of single user
        case MessageConstants.RECIPIENT_USER:
            list.addElement(_adminDomain.getUser(recipientName));
            break;

            // recipient is a user group, so a list of all individual
            //  users must be obtained
        case MessageConstants.RECIPIENT_USER_GROUP:
            Enumeration e = getAllMemberUsers(recipientName);
            while(e.hasMoreElements()) {
                list.addElement(e.nextElement());
            }
            break;

        default:
            System.out.println(getClass().getName()
                               + ".getRecipientList() found unknown type: "
                               + type);
            break;
        }

        return(list);
    }



    /**
     * Gives an enumeration of User objects, containing all members
     * of group 'groupName' and its subgroups.  Uses the recursive
     * method <em>getAllMemberUsersRecursive()</em> to descend all 
     * subgroup trees.
     * 
     * @param groupName
     * @return An enumeration of individual users which are the members
     *      of groupName.
     * @exception ResourceException
     */
    Enumeration getAllMemberUsers(String groupName) 
        throws ResourceException
    {
        return(getAllMemberUsersRecursive(groupName));
    }



    /**
     * Recursive method for getting User members from nested groups.
     * This is called from <em>getAllMemberUsers()</em>.
     * 
     * @param groupName
     * @return An enumeration of individual users which are the members
     *      of groupName.
     * @exception ResourceException
     */
    Enumeration getAllMemberUsersRecursive(String groupName) 
        throws ResourceException
    {
        Vector v = new Vector();

        // Obtain a Resource object from the domain using the groupName
        Resource rsc = _adminDomain.get(ResourceTypes.USER_GROUP, groupName);
        
        // Use the Resource object to obtain a ResourceAttributes object.
        //  This is a collection of individual ResourceAttribute objects.
        //  One of these ResourceAttribute objects for a group is the
        //  GroupResource.CHILDREN attribute.  This contains a list of all
        //  of the Resources of the groups children.
        ResourceAttributes ra = rsc.getResourceAttributes();
        ResourceAttribute children = ra.get(GroupResource.CHILDREN);

        // Get list of all the Resources of the groupName's children
        Enumeration e = children.getAll();
        
        // iterate through children adding individual users to output list
        //  and recursively searching child groups for individual users
        while(e.hasMoreElements()) {
            Resource child = (Resource) e.nextElement();

            switch(child.getResourceType()) {
                
                // One of groupName's children is an individual user
                //  so this user is added to the output list
            case ResourceTypes.USER:
                v.addElement(child);
                break;

                // One of groupName's children is another group, so
                //  it must be recursively searched for individual users
            case ResourceTypes.USER_GROUP:
            
                // get the children of the child group
                Enumeration grandchildren 
                    = getAllMemberUsersRecursive(child.getName());
                    
                // add the children of the child group to the output list
                while(grandchildren.hasMoreElements()) {
                    v.addElement(grandchildren.nextElement());
                }
                break;

            default:
                break;
            }
        }

        return(v.elements());
    }

    /**
     * Retrieve a PreferenceSet for the given user at the specified
     * context.
     * 
     * @param userId id of user to retrieve PreferenceSet for
     * @param preferenceContext 
     * @return A PreferenceSet object for the specified userId
     * @exception ResourceException
     */
    PreferenceSet getUserPreferenceSet(String userId, 
                                       String preferenceContext) 
        throws ResourceException
    {
        // get Resource associated with particular userId
        Resource user = _adminDomain.getUser(userId);
        
        // Use resource to obtain the PreferenceContext associated with 
        //  the user and preferenceContext string        
        PreferenceContext prefContext
            = ExtendedPreferenceContext.getInitialContextForResource(user, 
                                                           preferenceContext,
                                                           PreferenceConstants.CONTEXTONLY);

        // make the PreferenceContext up to date
        //  This call causes the client cache of preferences to be updated 
        //  to the current contents of the Network registry        
        prefContext.refresh();

        // create a working copy to save the PreferenceContext
        PreferenceSet prefs = new PreferenceSet();
        
        // load the current state of the PreferenceContext into the 
        //  PreferenceSet
        prefs.load(prefContext);

        return(prefs);
    }


    /**
     * Method used to carry out sending of message.  It determines
     * the preferred method of delivery, and takes corresponding 
     * action.
     * 
     * @param user individual recipient of message
     * @param message text of message to send
     *
     * @return a text response to indicate status.
     *
     * @exception java.io.IOException
     * @exception PersistenceException
     * @exception java.lang.ClassNotFoundException
     * @exception ResourceException
     */
    String sendMessageToUser(String user, String message) 
        throws IOException, PersistenceException, ClassNotFoundException,
               ResourceException
    {
        // Obtain a working copy of the individual users preferences
        //  This includes the preferred method for message delivery.
        PreferenceSet prefs 
            = getUserPreferenceSet(user, MessageConstants.PREFERENCE_CONTEXT);

        String notificationMethod;

        try {
            // Obtain the preferred notification method from the users
            //  PreferenseSet
            notificationMethod
                = (String) prefs.get(MessageConstants.IMMEDIATE_CONTACT);

        } catch(NoPreferenceException x) {
            notificationMethod = new String(MessageConstants.NOTIFY_NONE);
        }

        // Use different convenience method based on users preferred
        //  notification method
        if(notificationMethod.equals(MessageConstants.NOTIFY_PAGER)) {
            return(sendPageToUser(user, message, prefs));

        } else if(notificationMethod.equals(MessageConstants.NOTIFY_EMAIL)) {
            return(sendEmailToUser(user, message, prefs));

        } else if(notificationMethod.equals(MessageConstants.NOTIFY_FAX)) {
            return(sendFaxToUser(user, message, prefs));

        } else if(notificationMethod.equals(MessageConstants.NOTIFY_BACKUP)) {
            String backupUserId;

            try {
                backupUserId 
                    = (String) prefs.get(MessageConstants.BACKUP_USERID);

            } catch(NoPreferenceException x) {
                return("User " + user + ":  Message not sent because " 
                       + "notifcation specified as backup, "
                       + "but no backup user given.");
            }

            return(sendMessageToUser(backupUserId, 
                                     "For " + user + ": " + message));

        } else {
            return("User " + user + ":  Message not sent because " 
                + user + " has not specified a notification method.  "
                + "Cannot deliver message.");
        }
    }


    // *****************************************************************
    //
    // Convenience methods for sending messages via various notification
    // methods.
    //
    // *****************************************************************

    /**
     * Convenience method for sending a message via a page.     
     * 
     * @param user The user to receive the page.
     * @param message The text to send
     * @param prefs The user's preferences
     *
     * @return a text response to indicate status.
     */
    String sendPageToUser(String user, String message, PreferenceSet prefs)
    {
        String pagerType;
        String pagerPin;

        try {
            // use Preferences to get pager type
            pagerType = (String) prefs.get(MessageConstants.PAGER_TYPE);

        } catch(NoPreferenceException x) {
            pagerType = new String("None");
        }

        if(pagerType.equals("None")) {
            return("User " + user + ":  Message not sent because " 
                   + "notification specified as pager, but "
                   + "pager type not defined.");
        }

        try {
            // use Preferences to get pin number
            pagerPin = (String) prefs.get(MessageConstants.PAGER_PIN);

        } catch(NoPreferenceException x) {
            return("User " + user + ":  Message not sent because " 
                   + "notification specified as pager, but"
                   + " required PIN is not defined.");
        }

        
        PagerMessenger pm 
            = PagerMessengerFactory.getPagerMessenger(pagerType, pagerPin);

        if(pm.sendPage(message)) {
            return("User " + user + ":  Sent message to pager.");
        } else {
            return("User " + user + ":  Failed to send page.");
        }
    }


    /**
     * Convenience method for sending a message via email.
     * Does not actually send an email.
     * 
     * @param user The user to receive the email.
     * @param message The text to send.
     * @param prefs The user's preferences
     *
     * @return a text response to indicate status.     
     */
    String sendEmailToUser(String user, String message, PreferenceSet prefs) 
    {
        String emailAddress;

        try {
            emailAddress 
                = (String) prefs.get(MessageConstants.EMAIL_ADDRESS);

        } catch(NoPreferenceException x) {
            return("User " + user + ":  Message not sent because " 
                   + "notification specified as email, but"
                   + " email address not defined.");
        }

        return("User " + user + ":  To send as email to " + emailAddress);
    }


    /**
     * Convenience method for sending a message via fax.
     * Does not actually send a fax.
     * 
     * @param user The user to receive the fax.
     * @param message The text to send.
     * @param prefs The user's preferences
     *
     * @return a text response to indicate status.     
     */
    String sendFaxToUser(String user, String message, PreferenceSet prefs) 
    {
        String externalFax;

        try {
            externalFax 
                = (String) prefs.get(MessageConstants.EXTERNAL_FAX);

        } catch(NoPreferenceException x) {
            return("User " + user + ":  Message not sent because " 
                   + "notification specified as fax, but "
                   + "external fax number not defined.");
        }

        return("User " + user + ":  To send as fax to " + externalFax);
    }
}
