/*================================================================== 
 * Licensed Materials - Property of IBM 
 * IBM SecureWay On Demand Client/Server 
 * (c) Copyright IBM Corp. 1999  All rights reserved. 
 * 
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with IBM 
 * Corp. 
 * 
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is 
 * sample code created by IBM Corporation.  This sample code is 
 * not part of any standard or IBM product and is provided to you 
 * solely for the purpose of assisting you in the development of 
 * your applications.  The code is provided "AS IS", without 
 * warranty of any kind.  IBM shall not be liable for any damages 
 * arising out of your use of the sample code, even if they have 
 * been advised of the possibility of such damages. 
 ******************************************************************** 
 * SocksConfig.java
 *  
 * Demo: Pager Messaging 
 *
 * Purpose:
 * Simple servlet to configure the socks proxy for the Java TCP/IP stack.
 ******************************************************************** 
 * Change History: 
 ******************************************************************** 
 * 
 *  REASON   DATE     ORIGIN     DESCRIPTION 
 *  -------- ------   ------     ----------- 
 *  new      060899   Steve Ims  New code 
 *===================================================================*/ 

package Demos.MessageDemo.SocksConfig;

import java.io.*;
import java.net.*;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

/**
 * <p>
 * Simple servlet to configure the socks proxy for the Java TCP/IP stack.
 * 
 * <p>
 * All settings are stored permanently in a file $(dir1)\socks.properties.
 * The stored settings are restored on startup if this servlet is added to
 * the list of initialization servlets in $(dir2)\servlets.properties.
 * 
 * <blockquote>
 * $(dir1) = x:\WebSphere\AppServer\properties\server\servlet
 * $(dir2) = x:\WebSphere\AppServer\properties\server\servlet\servletservice
 * </blockquote>
 * 
 * <p>
 * Steps to enable automatic initialization.  
 * Modify $(dir2)\servlets.properties as follows...
 * <blockquote>
 * 1.  Create new entry for the SocksConfig servlet
 * 
 *    # Servlets added by the user
 *    servlet.ServerActivationServlet.code=com.ibm.managed.serverconfig.ServerActivationServlet
 *    servlet.CodebaseProxy.code=com.ibm.managed.codebaseservlet.CodebaseProxy
 *    servlet.SocksConfig.code=local.SocksConfig.SocksConfig
 * 
 * 
 * 2.  Add SocksConfig servlet to the list of startup servlets.
 *    # servlets to be loaded at startup ... 
 *    servlets.startup=invoker ServerActivationServlet SocksConfig
 * </blockquote>
 * 
 * @author Steve Ims
 * @see MessageServer
 */
public class SocksConfig extends HttpServlet 
{
    public void init(ServletConfig config) 
        throws ServletException
    {
        super.init(config);
        initializeSocks();
    }


    /**
     * <p>Attempt to setup socks according to host and port in the
     * <b>socks.properties</b> file.  If the file does not exist, or the
     * proxy is not reachable, then do nothing.
     */
    void initializeSocks() 
    {
        try {
            Properties systemProperties = System.getProperties();
        
            // look for socks properties file
            FileInputStream fis 
                = new FileInputStream(getSocksPropertiesFileName());
            Properties socksProperties = new Properties();
            socksProperties.load(fis);

            String socksProxyHost 
                = (String) socksProperties.get("socksProxyHost");
            String socksProxyPort 
                = (String) socksProperties.get("socksProxyPort");

            // If the socks proxy is reachable, set the system
            // properties accordingly.  Otherwise, do nothing.
            if(isProxyReachable(socksProxyHost, socksProxyPort)) {
                systemProperties.put("socksProxyHost", socksProxyHost);
                systemProperties.put("socksProxyPort", socksProxyPort);
                System.setProperties(systemProperties);
            }
                
        } catch(Exception x) {
            x.printStackTrace();
        }
    }

    /**
     * <p>Build <b>socks.properties</b> location string at:
     *      <server.root>\properties\server\servlet\socks.properties
     */
    String getSocksPropertiesFileName() 
    {
        Properties systemProperties = System.getProperties();

        String fileBase 
            = (String) systemProperties.get("server.root");
        String fileSep 
            = (String) systemProperties.get("file.separator");

        return(fileBase + fileSep
            + "properties" + fileSep
            + "server" + fileSep
            + "servlet" + fileSep
            + "socks.properties");
    }


    boolean isProxyReachable(String socksProxyHost, String socksProxyPort) 
    {
        try {
            
            // Test that the socks server is reachable.
            // open socket with host/port of proxy server
            Socket s = new Socket(InetAddress.getByName(socksProxyHost),
                                  Integer.parseInt(socksProxyPort));
            s.close();

            return(true);

        } catch(Exception x) {
            System.out.println(getClass().getName() + " couldn't reach socks "
                               + "proxy " + socksProxyHost + " at port "
                               + socksProxyPort + ".");

            return(false);
        }
    }


    /**
     * Show current state of socks configuration, and display
     * the form to configure socks.
     *
     * @param req
     * @param res
     * @exception java.io.IOException
     */
    public void doGet(HttpServletRequest req, HttpServletResponse res)
        throws ServletException, IOException
    {
        boolean enabled = true;
        Properties systemProperties = System.getProperties();

        String socksProxyHost 
            = (String) systemProperties.get("socksProxyHost");
        if(socksProxyHost == null) {
            socksProxyHost = "";
            enabled = false;
        }

        String socksProxyPort 
            = (String) systemProperties.get("socksProxyPort");
        if(socksProxyPort == null) {
            socksProxyPort = "";
        }

        String msg 
            = "Socks is " + (enabled ? "" : "not") + " currently enabled.";

        showSocksInfo(req, res, socksProxyHost, socksProxyPort, msg);
    }

    /**
     * <p>Called when the servlet form is submitted, this method sets up
     * the system properties <em>socksProxyHost</em> amd <em>socksProxyPort</em>.
     * It also writes the two system properties in a file called 
     * <b>socks.properties</b>.
     * 
     * @param req
     * @param res
     * @exception java.io.IOException
     */
    public void doPost(HttpServletRequest req, HttpServletResponse res)
        throws ServletException, IOException
    {
        String [] enable = req.getParameterValues("Enable");
        Properties systemProperties = System.getProperties();
        String msg;

        // if enable button not clicked, then disable socks support
        if(enable == null) {
            systemProperties.remove("socksProxyHost");
            systemProperties.remove("socksProxyPort");
            msg = "Socks is not currently enabled.";

        } else {
        // enable socks support 
        
            // extract host and port from post
            String socksProxyHost 
                = req.getParameterValues("socksProxyHost")[0];
            String socksProxyPort 
                = req.getParameterValues("socksProxyPort")[0];
            
            // test if proxy is reachable before setting system values
            if(isProxyReachable(socksProxyHost, socksProxyPort)) {
                systemProperties.put("socksProxyHost", socksProxyHost);
                systemProperties.put("socksProxyPort", socksProxyPort);
                msg = "Socks is currently enabled.";

            } else {
                systemProperties.remove("socksProxyHost");
                systemProperties.remove("socksProxyPort");
                msg = "Socks be not enabled because could not reach socks proxy " 
                    + socksProxyHost + " at port " + socksProxyPort + ".";
            }
        }
        
        System.setProperties(systemProperties);

        // write socks settings to a file
        writeSocksPropertiesFile();
        showSocksInfo(req, res, 
                      (String) systemProperties.get("socksProxyHost"),
                      (String) systemProperties.get("socksProxyPort"),
                      msg);
    }

    /**
     * <p>Write socks Proxy info to the file <b>socks.properties</b> .
     *    The file will be located at:
     *          <server.root>\properties\server\servlet\socks.properties
     */
    void writeSocksPropertiesFile()
    {
        try {
            Properties systemProperties = System.getProperties();
            Properties socksProperties = new Properties();

            String socksProxyHost 
                = (String) systemProperties.get("socksProxyHost");
            String socksProxyPort 
                = (String) systemProperties.get("socksProxyPort");

            if(socksProxyHost != null) {
                socksProperties.put("socksProxyHost", socksProxyHost);
                socksProperties.put("socksProxyPort", socksProxyPort);
            }

            FileOutputStream fos 
                = new FileOutputStream(getSocksPropertiesFileName());

            String header = "Socks Configuration Parameters";
            socksProperties.save(fos, header);

        } catch(Exception x) {
            x.printStackTrace();
        }
    }

    /**
     * <p> Display the socks configuration form.
     * 
     * @param req
     * @param res
     * @exception java.io.IOException
     */
    void showSocksInfo(HttpServletRequest req, HttpServletResponse res,
                       String socksProxyHost, String socksProxyPort,
                       String message)
        throws ServletException, IOException
    {
        Properties systemProperties = System.getProperties();
        
        // Build HTML form
	    PrintWriter		out;

	    res.setContentType("text/html");
	    out = res.getWriter();

        out.println("<html>");
        out.println("<head>");
        out.println("<title>");
        out.println("WebSphere Application Server Socks Configuration");
        out.println("</title>");
        out.println("</head>");

        out.println("<body>");

        out.println("<b>" + message + "</b>");

        out.println("<hr>");

        out.println("<form action=/servlet/local.SocksConfig.SocksConfig "
                    + "method=POST>");
        out.println("<table>");
        out.println("<tr><td>Socks Proxy Host</td>"
                    + "<td><input type=text name=socksProxyHost value=" 
                    + ((socksProxyHost != null) ? socksProxyHost : "")
                    + "></td></tr>");
        out.println("<tr><td>Socks Proxy Port</td>"
                    + "<td><input type=text name=socksProxyPort value="
                    + ((socksProxyPort != null) ? socksProxyPort : "")
                    + "></td></tr>");
        out.println("</table>");
        out.println("<input type=submit name=Enable value=Set>");
        out.println("<input type=submit name=Disable value=Disable>");
        out.println("</form>");

        out.println("</body>");
        out.println("</html>");
        
        out.close();
    }
}
